"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLambdaFunction = void 0;
const taquito_1 = require("@mavrykdynamics/taquito");
const child_process_1 = require("child_process");
const fs = require("fs");
const path = require("path"); // Import the 'path' module for path handling
const lambdaFunctionLibrary_1 = require("./lambdaFunctionLibrary");
const packLambdaFunction = (tezos, governanceProxyContractAddress, lambdaFunction) => __awaiter(void 0, void 0, void 0, function* () {
    const governanceProxyInstance = yield tezos.contract.at(governanceProxyContractAddress);
    const param = governanceProxyInstance.methods.dataPackingHelper(lambdaFunction).toTransferParams();
    if (param.parameter) {
        const paramValue = param.parameter.value;
        const lambdaEntrypointType = yield governanceProxyInstance.entrypoints.entrypoints.dataPackingHelper;
        const packed = yield tezos.rpc.packData({
            data: paramValue,
            type: lambdaEntrypointType
        }).catch(e => console.error('error:', e));
        var packedParam;
        if (packed) {
            packedParam = packed.packed;
        }
        else {
            throw `packing failed`;
        }
    }
    return packedParam;
});
function getLigo(isDockerizedLigo, ligoVersion) {
    const pwd = process.cwd();
    let pathToLigo = 'ligo';
    if (isDockerizedLigo) {
        pathToLigo = `docker run --platform=linux/amd64 -v ${pwd}:${pwd} -w ${pwd} --rm -i mavrykdynamics/ligo:${ligoVersion}`;
    }
    try {
        (0, child_process_1.execSync)(`${pathToLigo} --help`);
    }
    catch (err) {
        console.error(err);
        throw new Error('Failed to execute LIGO compiler');
    }
    return pathToLigo;
}
const compileLambdaFunctionContract = (contractPath = "", ligoVersion = "0.60.0") => __awaiter(void 0, void 0, void 0, function* () {
    const ligo = getLigo(true, ligoVersion);
    const jsonFormat = (0, child_process_1.execSync)(`${ligo} compile contract "${contractPath}" --michelson-format json --protocol kathmandu`, {
        maxBuffer: 1024 * 1024,
        timeout: 1024 * 1024,
    }).toString();
    // Get the lambda function from the compiled code
    return JSON.parse(jsonFormat)[2].args[0][1].args[0];
});
const getLambdaFunction = (rpc, governanceProxyContractAddress, lambdaFunctionName, lambdaFunctionParameters = []) => __awaiter(void 0, void 0, void 0, function* () {
    // Generate the contract code
    const generatedContract = (0, lambdaFunctionLibrary_1.generateProxyContract)(lambdaFunctionName, lambdaFunctionParameters);
    // Write the result to the output file
    const outputFile = path.resolve(process.cwd(), "governanceProxyLambdaFunction.ligo");
    fs.writeFileSync(outputFile, generatedContract);
    // Start the compiling process
    const lambdaFunctionJson = yield compileLambdaFunctionContract(outputFile);
    const tezos = new taquito_1.TezosToolkit(rpc);
    const packedLambdaFunction = yield packLambdaFunction(tezos, governanceProxyContractAddress, lambdaFunctionJson);
    // Reset the file
    fs.writeFileSync(outputFile, "");
    return packedLambdaFunction;
});
exports.getLambdaFunction = getLambdaFunction;
